/*
 * pio_blink.ino
 *
 * This program demonstrates how to use the Programmable Input/Output (PIO) feature 
 * of the RP2040 microcontroller within an Arduino MbedOS environment. 
 * It initializes multiple state machines to blink LEDs at different frequencies 
 * using a custom PIO program. Each LED frequency is configurable via the function arguments.
 *
 * Features:
 * - Configures three GPIO pins (GPIO6, GPIO7, GPIO8) for LED control.
 * - Uses RP2040's PIO hardware to achieve high-precision timing.
 * - Allows flexible frequency adjustment for each LED.
 * 
 * Dependencies:
 * - RP2040 hardware clocks and PIO libraries (from MbedOS environment).
 * - blink.pio.h: A generated header file containing the compiled PIO assembly program.
 *
 */



#include "hardware/clocks.h"
#include "hardware/pio.h"
#include "blink.pio.h"                 // Generált PIO fejlécfájl

void blink_pin_forever(PIO pio,uint sm,uint offset,uint pin,uint freq) {
    blink_program_init(pio, sm, offset, pin);
    pio_sm_set_enabled(pio, sm, true);
    uint32_t delay_cycles = (clock_get_hz(clk_sys) / (2 * freq)) - 3;
    pio_sm_put(pio, sm, delay_cycles); // Send period to FIFO
}

void setup() {
    PIO pio = pio0;
    uint offset = pio_add_program(pio, &blink_program);
    blink_pin_forever(pio, 0, offset, 6, 3); // GPIO6 3 Hz
    blink_pin_forever(pio, 1, offset, 7, 4); // GPIO7 4 Hz
    blink_pin_forever(pio, 2, offset, 8, 1); // GPIO8 1 Hz
}

void loop() {
  // Do nothing
}
