#include "hardware/clocks.h"
#include "hardware/pio.h"

// PIO program LED villogtatáshoz
// Az utasításokat egy statikus tömbbe helyezzük
static const uint16_t blink_instructions[] = {
    pio_encode_pull(false, true),     // 0: pull   block
    pio_encode_out(pio_y, 32),        // 1: out    y, 32
                                      // .wrap_target
    pio_encode_mov(pio_x, pio_y),     // 2: mov    x, y
    pio_encode_set(pio_pins, 1),      // 3: set    pins, 1
    pio_encode_jmp_x_dec(4),          // 4: jmp    x--, 4
    pio_encode_mov(pio_x, pio_y),     // 5: mov    x, y
    pio_encode_set(pio_pins, 0),      // 6: set    pins, 0
    pio_encode_jmp_x_dec(7)           // 7: jmp    x--, 7
                                      // .wrap
};

// A PIO program immár a statikus utasítástömbre mutat
const pio_program_t blink_program = {
  .instructions = blink_instructions,
  .length = 8,                          // Az utasítások száma
  .origin = -1 
};

void blink_program_init(PIO pio, uint sm, uint offset, uint pin) {
  pio_gpio_init(pio, pin);
  pio_sm_set_consecutive_pindirs(pio, sm, pin, 1, true);
  pio_sm_config c = pio_get_default_sm_config();
  sm_config_set_wrap(&c, offset+2, offset+7);  // Wrap címek megadása
  sm_config_set_set_pins(&c, pin, 1);
  pio_sm_init(pio, sm, offset, &c);
}

void blink_pin_forever(PIO pio, uint sm, uint offset, uint pin, uint freq) {
  blink_program_init(pio, sm, offset, pin);
  pio_sm_set_enabled(pio, sm, true);
  uint32_t delay_cycles = (clock_get_hz(clk_sys) / (2 * freq)) - 3;
  pio_sm_put(pio, sm, delay_cycles); // Send period to FIFO
}

void setup() {
  PIO pio = pio0;
  uint offset = pio_add_program(pio, &blink_program);
  blink_pin_forever(pio, 0, offset, 6, 3); // GPIO6 3 Hz
  blink_pin_forever(pio, 1, offset, 7, 4); // GPIO7 4 Hz
  blink_pin_forever(pio, 2, offset, 8, 1); // GPIO8 1 Hz
}

void loop() {
  // Do nothing
}
