
#include "hardware/pio.h"
#include "hardware/clocks.h"
#define LED_PIN 16
PIO pio = pio0;
uint sm = 0;

static const uint16_t instructions[] = {
  0x6221, // 0: out x, 1 side 0 [2]
  0x1123, // 1: jmp !x, 3 side 1 [1]
  0x1400, // 2: jmp 0 side 1 [4]
  0xa442, // 3: nop side 0 [2]
};

pio_program_t ws2812_program = {
  .instructions = instructions,
  .length = 4,
  .origin = -1,
};

void ws2812_program_init(PIO pio, uint sm, uint offset, uint pin, float freq) {
  pio_gpio_init(pio, pin);
  pio_sm_set_consecutive_pindirs(pio, sm, pin, 1, true);
  pio_sm_config c = pio_get_default_sm_config();
  sm_config_set_wrap(&c, offset, offset + ws2812_program.length - 1);
  sm_config_set_sideset(&c, 1, false, false);
  sm_config_set_sideset_pins(&c, pin);
  sm_config_set_out_shift(&c, false, true, 24); // 24 bites adatküldés
  sm_config_set_fifo_join(&c, PIO_FIFO_JOIN_TX);
  int cycles_per_bit = 10; // T1 + T2 + T3
  float div = clock_get_hz(clk_sys) / (freq * cycles_per_bit);
  sm_config_set_clkdiv(&c, div);
  pio_sm_init(pio, sm, offset, &c);
  pio_sm_set_enabled(pio, sm, true);
}

void setup() {
  uint offset = pio_add_program(pio, &ws2812_program);
  ws2812_program_init(pio, sm, offset, LED_PIN, 800000);
}

void loop() {
  pio_sm_put_blocking(pio, sm, 0x00FF0000); // Piros
  delay(1000);
  pio_sm_put_blocking(pio, sm, 0xFF000000); // Zöld
  delay(1000);
  pio_sm_put_blocking(pio, sm, 0x0000FF00); // Kék
  delay(1000);
  pio_sm_put_blocking(pio, sm, 0x00000000); // LED ki
  delay(1000);
}
